@@ -10,7 +10,7 @@
 /*
  * (C) 2001,2005 by Marcin Dalecki <martin@dalecki.de>
  *
- * Implementation of dialogue functions for the Motif GUI variant.
+ * Implementation of dialog functions for the Motif GUI variant.
  *
  * Note about Lesstif: Apparently lesstif doesn't get the widget layout right,
  * when using a dynamic scrollbar policy.
@@ -168,7 +168,7 @@ name_part(char *font, char *buf)
     get_part(font, 2, buf2);
     get_part(font, 1, buf3);
 
-    if (strlen(buf3))
+    if (*buf3 != NUL)
 	vim_snprintf(buf, TEMP_BUF_SIZE, "%s (%s)", buf2, buf3);
     else
 	vim_snprintf(buf, TEMP_BUF_SIZE, "%s", buf2);
@@ -225,7 +225,7 @@ size_part(char *font, char *buf, int inPixels)
     if (inPixels)
     {
 	get_part(font, 7, buf);
-	if (strlen(buf) > 0)
+	if (*buf != NUL)
 	{
 	    size = atoi(buf);
 	    sprintf(buf, "%3d", size);
@@ -234,7 +234,7 @@ size_part(char *font, char *buf, int inPixels)
     else
     {
 	get_part(font, 8, buf);
-	if (strlen(buf) > 0)
+	if (*buf != NUL)
 	{
 	    size = atoi(buf);
 	    temp = (float)size / 10.0;
@@ -261,7 +261,7 @@ encoding_part(char *font, char *buf)
     get_part(font, 13, buf1);
     get_part(font, 14, buf2);
 
-    if (strlen(buf1) > 0 && strlen(buf2))
+    if (*buf1 != NUL && *buf2 != NUL)
 	vim_snprintf(buf, TEMP_BUF_SIZE, "%s-%s", buf1, buf2);
     if (!strcmp(buf, " "))
 	strcpy(buf, "-");
@@ -633,16 +633,19 @@ do_choice(Widget w,
 	data->sel[which] = XtNewString(sel);
     else
     {
-	XtFree(data->sel[which]);
 	if (!strcmp(data->sel[which], sel))
 	{
 	    /* unselecting current selection */
+	    XtFree(data->sel[which]);
 	    data->sel[which] = NULL;
 	    if (w)
 		XmListDeselectItem(w, call_data->item);
 	}
 	else
+	{
+	    XtFree(data->sel[which]);
 	    data->sel[which] = XtNewString(sel);
+	}
     }
     XtFree(sel);
 
@@ -685,7 +688,7 @@ do_choice(Widget w,
 	    do_dialog(VIM_ERROR,
 		    (char_u *)_("Error"),
 		    (char_u *)_("Invalid font specification"),
-		    (char_u *)_("&Dismiss"), 1, NULL);
+		    (char_u *)_("&Dismiss"), 1, NULL, FALSE);
 
 	return True;
     }
@@ -804,7 +807,7 @@ ok_callback(Widget w UNUSED,
 	do_dialog(VIM_ERROR,
 		(char_u *)_("Error"),
 		(char_u *)_("Invalid font specification"),
-		(char_u *)_("&Dismiss"), 1, NULL);
+		(char_u *)_("&Dismiss"), 1, NULL, FALSE);
 	XFreeFontNames(name);
     }
     else
@@ -1226,10 +1229,10 @@ gui_xm_select_font(char_u *current)
 	    size_part(found, sizebuf, data->in_pixels);
 	    encoding_part(found, encodingbuf);
 
-	    if (strlen(namebuf) > 0
-		    && strlen(stylebuf) > 0
-		    && strlen(sizebuf) > 0
-		    && strlen(encodingbuf) > 0)
+	    if (*namebuf != NUL
+		    && *stylebuf != NUL
+		    && *sizebuf != NUL
+		    && *encodingbuf != NUL)
 	    {
 		data->sel[NAME] = XtNewString(namebuf);
 		data->sel[STYLE] = XtNewString(stylebuf);
@@ -1271,13 +1274,12 @@ gui_xm_select_font(char_u *current)
 	XtAppProcessEvent(XtWidgetToApplicationContext(data->dialog),
 							(XtInputMask)XtIMAll);
 
-    XtDestroyWidget(data->dialog);
-
     if (data->old)
     {
 	XFreeFont(XtDisplay(data->dialog),  data->old);
 	XmFontListFree(data->old_list);
     }
+    XtDestroyWidget(data->dialog);
 
     gui_motif_synch_fonts();
 
