@@ -170,10 +170,12 @@ static int mzscheme_init(void);
 #ifdef FEAT_EVAL
 static Scheme_Object *vim_to_mzscheme(typval_T *vim_value, int depth,
 	Scheme_Hash_Table *visited);
+static int mzscheme_to_vim(Scheme_Object *obj, typval_T *tv, int depth,
+	Scheme_Hash_Table *visited);
 #endif
 
 #ifdef MZ_PRECISE_GC
-static int buffer_size_proc(void *obj)
+static int buffer_size_proc(void *obj UNUSED)
 {
     return gcBYTES_TO_WORDS(sizeof(vim_mz_buffer));
 }
@@ -185,7 +187,7 @@ static int buffer_fixup_proc(void *obj)
 {
     return buffer_size_proc(obj);
 }
-static int window_size_proc(void *obj)
+static int window_size_proc(void *obj UNUSED)
 {
     return gcBYTES_TO_WORDS(sizeof(vim_mz_window));
 }
@@ -554,8 +556,8 @@ mzscheme_runtime_link_init(char *sch_dll, char *gc_dll, int verbose)
 
     if (hMzGC && hMzSch)
 	return OK;
-    hMzSch = LoadLibrary(sch_dll);
-    hMzGC = LoadLibrary(gc_dll);
+    hMzSch = vimLoadLib(sch_dll);
+    hMzGC = vimLoadLib(gc_dll);
 
     if (!hMzSch)
     {
@@ -706,16 +708,16 @@ static void remove_timer(void);
 /* timers are presented in GUI only */
 # if defined(FEAT_GUI_W32)
     static void CALLBACK
-timer_proc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime)
+timer_proc(HWND hwnd UNUSED, UINT uMsg UNUSED, UINT idEvent UNUSED, DWORD dwTime UNUSED)
 # elif defined(FEAT_GUI_GTK)
     static gint
-timer_proc(gpointer data)
+timer_proc(gpointer data UNUSED)
 # elif defined(FEAT_GUI_MOTIF) || defined(FEAT_GUI_ATHENA)
     static void
-timer_proc(XtPointer timed_out, XtIntervalId *interval_id)
+timer_proc(XtPointer timed_out UNUSED, XtIntervalId *interval_id UNUSED)
 # elif defined(FEAT_GUI_MAC)
     pascal void
-timer_proc(EventLoopTimerRef theTimer, void *userData)
+timer_proc(EventLoopTimerRef theTimer UNUSED, void *userData UNUSED)
 # endif
 {
     scheme_check_threads();
@@ -792,9 +794,16 @@ mzscheme_end(void)
 #endif
 }
 
+#if MZSCHEME_VERSION_MAJOR >= 500 && defined(WIN32) && defined(USE_THREAD_LOCAL)
+static __declspec(thread) void *tls_space;
+#endif
+
     void
 mzscheme_main(void)
 {
+#if MZSCHEME_VERSION_MAJOR >= 500 && defined(WIN32) && defined(USE_THREAD_LOCAL)
+    scheme_register_tls_space(&tls_space, 0);
+#endif
 #if defined(MZ_PRECISE_GC) && MZSCHEME_VERSION_MAJOR >= 400
     /* use trampoline for precise GC in MzScheme >= 4.x */
     scheme_main_setup(TRUE, mzscheme_env_main, 0, NULL);
@@ -804,7 +813,7 @@ mzscheme_main(void)
 }
 
     static int
-mzscheme_env_main(Scheme_Env *env, int argc, char **argv)
+mzscheme_env_main(Scheme_Env *env, int argc UNUSED, char **argv UNUSED)
 {
     /* neither argument nor return values are used */
 #ifdef MZ_PRECISE_GC
@@ -1188,7 +1197,7 @@ ex_mzscheme(exarg_T *eap)
 }
 
     static Scheme_Object *
-do_load(void *data, int noargc, Scheme_Object **noargv)
+do_load(void *data, int noargc UNUSED, Scheme_Object **noargv UNUSED)
 {
     Scheme_Object   *expr = NULL;
     Scheme_Object   *result = NULL;
@@ -1304,7 +1313,7 @@ extract_exn_message(Scheme_Object *v)
 }
 
     static Scheme_Object *
-do_eval(void *s, int noargc, Scheme_Object **noargv)
+do_eval(void *s, int noargc UNUSED, Scheme_Object **noargv UNUSED)
 {
     return scheme_eval_string_all((char *)s, environment, TRUE);
 }
@@ -1315,7 +1324,7 @@ do_eval(void *s, int noargc, Scheme_Object **noargv)
  *========================================================================
  */
     static void
-do_intrnl_output(char *mesg, long len, int error)
+do_intrnl_output(char *mesg, int error)
 {
     char *p, *prev;
 
@@ -1339,21 +1348,21 @@ do_intrnl_output(char *mesg, long len, int error)
 }
 
     static void
-do_output(char *mesg, long len)
+do_output(char *mesg, long len UNUSED)
 {
-    do_intrnl_output(mesg, len, 0);
+    do_intrnl_output(mesg, 0);
 }
 
     static void
-do_err_output(char *mesg, long len)
+do_err_output(char *mesg)
 {
-    do_intrnl_output(mesg, len, 1);
+    do_intrnl_output(mesg, 1);
 }
 
     static void
 do_printf(char *format, ...)
 {
-    do_intrnl_output(format, STRLEN(format), 1);
+    do_intrnl_output(format, 1);
 }
 
     static void
@@ -1366,7 +1375,7 @@ do_flush(void)
     MZ_GC_CHECK();
     if (length)
     {
-	do_err_output(buff, length);
+	do_err_output(buff);
 	return;
     }
 
@@ -1436,21 +1445,21 @@ vim_eval(void *data, int argc, Scheme_Object **argv)
 
 /* (range-start) */
     static Scheme_Object *
-get_range_start(void *data, int argc, Scheme_Object **argv)
+get_range_start(void *data UNUSED, int argc UNUSED, Scheme_Object **argv UNUSED)
 {
     return scheme_make_integer(range_start);
 }
 
 /* (range-end) */
     static Scheme_Object *
-get_range_end(void *data, int argc, Scheme_Object **argv)
+get_range_end(void *data UNUSED, int argc UNUSED, Scheme_Object **argv UNUSED)
 {
     return scheme_make_integer(range_end);
 }
 
 /* (beep) */
     static Scheme_Object *
-mzscheme_beep(void *data, int argc, Scheme_Object **argv)
+mzscheme_beep(void *data UNUSED, int argc UNUSED, Scheme_Object **argv UNUSED)
 {
     vim_beep();
     return scheme_void;
@@ -1582,14 +1591,14 @@ set_option(void *data, int argc, Scheme_Object **argv)
 
 /* (curr-win) */
     static Scheme_Object *
-get_curr_win(void *data, int argc, Scheme_Object **argv)
+get_curr_win(void *data UNUSED, int argc UNUSED, Scheme_Object **argv UNUSED)
 {
     return (Scheme_Object *)get_vim_curr_window();
 }
 
 /* (win-count) */
     static Scheme_Object *
-get_window_count(void *data, int argc, Scheme_Object **argv)
+get_window_count(void *data UNUSED, int argc UNUSED, Scheme_Object **argv UNUSED)
 {
     win_T   *w;
     int	    n = 0;
@@ -1924,7 +1933,7 @@ get_buffer_num(void *data, int argc, Scheme_Object **argv)
 
 /* (buff-count) */
     static Scheme_Object *
-get_buffer_count(void *data, int argc, Scheme_Object **argv)
+get_buffer_count(void *data UNUSED, int argc UNUSED, Scheme_Object **argv UNUSED)
 {
     buf_T   *b;
     int	    n = 0;
@@ -1945,7 +1954,7 @@ get_buffer_name(void *data, int argc, Scheme_Object **argv)
 
 /* (curr-buff) */
     static Scheme_Object *
-get_curr_buffer(void *data, int argc, Scheme_Object **argv)
+get_curr_buffer(void *data UNUSED, int argc UNUSED, Scheme_Object **argv UNUSED)
 {
     return (Scheme_Object *)get_vim_curr_buffer();
 }
@@ -2519,7 +2528,7 @@ insert_buffer_line_list(void *data, int argc, Scheme_Object **argv)
  */
 /* (buff? obj) */
     static Scheme_Object *
-vim_bufferp(void *data, int argc, Scheme_Object **argv)
+vim_bufferp(void *data UNUSED, int argc UNUSED, Scheme_Object **argv)
 {
     if (SCHEME_VIMBUFFERP(argv[0]))
 	return scheme_true;
@@ -2529,7 +2538,7 @@ vim_bufferp(void *data, int argc, Scheme_Object **argv)
 
 /* (win? obj) */
     static Scheme_Object *
-vim_windowp(void *data, int argc, Scheme_Object **argv)
+vim_windowp(void *data UNUSED, int argc UNUSED, Scheme_Object **argv)
 {
     if (SCHEME_VIMWINDOWP(argv[0]))
 	return scheme_true;
@@ -2539,7 +2548,7 @@ vim_windowp(void *data, int argc, Scheme_Object **argv)
 
 /* (buff-valid? obj) */
     static Scheme_Object *
-vim_buffer_validp(void *data, int argc, Scheme_Object **argv)
+vim_buffer_validp(void *data UNUSED, int argc UNUSED, Scheme_Object **argv)
 {
     if (SCHEME_VIMBUFFERP(argv[0])
 	    && ((vim_mz_buffer *)argv[0])->buf != INVALID_BUFFER_VALUE)
@@ -2550,7 +2559,7 @@ vim_buffer_validp(void *data, int argc, Scheme_Object **argv)
 
 /* (win-valid? obj) */
     static Scheme_Object *
-vim_window_validp(void *data, int argc, Scheme_Object **argv)
+vim_window_validp(void *data UNUSED, int argc UNUSED, Scheme_Object **argv)
 {
     if (SCHEME_VIMWINDOWP(argv[0])
 	    && ((vim_mz_window *)argv[0])->win != INVALID_WINDOW_VALUE)
@@ -2733,6 +2742,225 @@ vim_to_mzscheme(typval_T *vim_value, int depth, Scheme_Hash_Table *visited)
     MZ_GC_UNREG();
     return result;
 }
+
+    static int
+mzscheme_to_vim(Scheme_Object *obj, typval_T *tv, int depth,
+	Scheme_Hash_Table *visited)
+{
+    int		status = OK;
+    typval_T	*found;
+    MZ_GC_CHECK();
+    if (depth > 100) /* limit the deepest recursion level */
+    {
+	tv->v_type = VAR_NUMBER;
+	tv->vval.v_number = 0;
+	return FAIL;
+    }
+
+    found = (typval_T *)scheme_hash_get(visited, obj);
+    if (found != NULL)
+	copy_tv(found, tv);
+    else if (SCHEME_VOIDP(obj))
+    {
+	tv->v_type = VAR_NUMBER;
+	tv->vval.v_number = 0;
+    }
+    else if (SCHEME_INTP(obj))
+    {
+	tv->v_type = VAR_NUMBER;
+	tv->vval.v_number = SCHEME_INT_VAL(obj);
+    }
+    else if (SCHEME_BOOLP(obj))
+    {
+	tv->v_type = VAR_NUMBER;
+	tv->vval.v_number = SCHEME_TRUEP(obj);
+    }
+# ifdef FEAT_FLOAT
+    else if (SCHEME_DBLP(obj))
+    {
+	tv->v_type = VAR_FLOAT;
+	tv->vval.v_float = SCHEME_DBL_VAL(obj);
+    }
+# endif
+    else if (SCHEME_STRINGP(obj))
+    {
+	tv->v_type = VAR_STRING;
+	tv->vval.v_string = vim_strsave((char_u *)SCHEME_STR_VAL(obj));
+    }
+    else if (SCHEME_VECTORP(obj) || SCHEME_NULLP(obj)
+	    || SCHEME_PAIRP(obj) || SCHEME_MUTABLE_PAIRP(obj))
+    {
+	list_T  *list = list_alloc();
+	if (list == NULL)
+	    status = FAIL;
+	else
+	{
+	    int		    i;
+	    Scheme_Object   *curr = NULL;
+	    Scheme_Object   *cval = NULL;
+	    /* temporary var to hold current element of vectors and pairs */
+	    typval_T	    *v;
+
+	    MZ_GC_DECL_REG(2);
+	    MZ_GC_VAR_IN_REG(0, curr);
+	    MZ_GC_VAR_IN_REG(1, cval);
+	    MZ_GC_REG();
+
+	    tv->v_type = VAR_LIST;
+	    tv->vval.v_list = list;
+	    ++list->lv_refcount;
+
+	    v = (typval_T *)alloc(sizeof(typval_T));
+	    if (v == NULL)
+		status = FAIL;
+	    else
+	    {
+		/* add the value in advance to allow handling of self-referencial
+		 * data structures */
+		typval_T    *visited_tv = (typval_T *)alloc(sizeof(typval_T));
+		copy_tv(tv, visited_tv);
+		scheme_hash_set(visited, obj, (Scheme_Object *)visited_tv);
+
+		if (SCHEME_VECTORP(obj))
+		{
+		    for (i = 0; i < SCHEME_VEC_SIZE(obj); ++i)
+		    {
+			cval = SCHEME_VEC_ELS(obj)[i];
+			status = mzscheme_to_vim(cval, v, depth + 1, visited);
+			if (status == FAIL)
+			    break;
+			status = list_append_tv(list, v);
+			clear_tv(v);
+			if (status == FAIL)
+			    break;
+		    }
+		}
+		else if (SCHEME_PAIRP(obj) || SCHEME_MUTABLE_PAIRP(obj))
+		{
+		    for (curr = obj;
+			    SCHEME_PAIRP(curr) || SCHEME_MUTABLE_PAIRP(curr);
+			    curr = SCHEME_CDR(curr))
+		    {
+			cval = SCHEME_CAR(curr);
+			status = mzscheme_to_vim(cval, v, depth + 1, visited);
+			if (status == FAIL)
+			    break;
+			status = list_append_tv(list, v);
+			clear_tv(v);
+			if (status == FAIL)
+			    break;
+		    }
+		    /* impoper list not terminated with null
+		     * need to handle the last element */
+		    if (status == OK && !SCHEME_NULLP(curr))
+		    {
+			status = mzscheme_to_vim(cval, v, depth + 1, visited);
+			if (status == OK)
+			{
+			    status = list_append_tv(list, v);
+			    clear_tv(v);
+			}
+		    }
+		}
+		/* nothing to do for scheme_null */
+		vim_free(v);
+	    }
+	    MZ_GC_UNREG();
+	}
+    }
+    else if (SCHEME_HASHTP(obj))
+    {
+	int		i;
+	dict_T		*dict;
+	Scheme_Object   *key = NULL;
+	Scheme_Object   *val = NULL;
+
+	MZ_GC_DECL_REG(2);
+	MZ_GC_VAR_IN_REG(0, key);
+	MZ_GC_VAR_IN_REG(1, val);
+	MZ_GC_REG();
+
+	dict = dict_alloc();
+	if (dict == NULL)
+	    status = FAIL;
+	else
+	{
+	    typval_T    *visited_tv = (typval_T *)alloc(sizeof(typval_T));
+
+	    tv->v_type = VAR_DICT;
+	    tv->vval.v_dict = dict;
+	    ++dict->dv_refcount;
+
+	    copy_tv(tv, visited_tv);
+	    scheme_hash_set(visited, obj, (Scheme_Object *)visited_tv);
+
+	    for (i = 0; i < ((Scheme_Hash_Table *)obj)->size; ++i)
+	    {
+		if (((Scheme_Hash_Table *) obj)->vals[i] != NULL)
+		{
+		    /* generate item for `diplay'ed Scheme key */
+		    dictitem_T  *item = dictitem_alloc((char_u *)string_to_line(
+				((Scheme_Hash_Table *) obj)->keys[i]));
+		    /* convert Scheme val to Vim and add it to the dict */
+		    if (mzscheme_to_vim(((Scheme_Hash_Table *) obj)->vals[i],
+				    &item->di_tv, depth + 1, visited) == FAIL
+			    || dict_add(dict, item) == FAIL)
+		    {
+			dictitem_free(item);
+			status = FAIL;
+			break;
+		    }
+		}
+
+	    }
+	}
+	MZ_GC_UNREG();
+    }
+    else
+    {
+	/* `display' any other value to string */
+	tv->v_type = VAR_STRING;
+	tv->vval.v_string = (char_u *)string_to_line(obj);
+    }
+    return status;
+}
+
+    void
+do_mzeval(char_u *str, typval_T *rettv)
+{
+    int i;
+    Scheme_Object	*ret = NULL;
+    Scheme_Hash_Table	*visited = NULL;
+
+    MZ_GC_DECL_REG(2);
+    MZ_GC_VAR_IN_REG(0, ret);
+    MZ_GC_VAR_IN_REG(0, visited);
+    MZ_GC_REG();
+
+    if (mzscheme_init())
+    {
+	MZ_GC_UNREG();
+	return;
+    }
+
+    MZ_GC_CHECK();
+    visited = scheme_make_hash_table(SCHEME_hash_ptr);
+    MZ_GC_CHECK();
+
+    if (eval_with_exn_handling(str, do_eval, &ret) == OK)
+	mzscheme_to_vim(ret, rettv, 1, visited);
+
+    for (i = 0; i < visited->size; ++i)
+    {
+	/* free up remembered objects */
+	if (visited->vals[i] != NULL)
+	{
+	    free_tv((typval_T *)visited->vals[i]);
+	}
+    }
+
+    MZ_GC_UNREG();
+}
 #endif
 
 /*
@@ -3094,7 +3322,7 @@ sandbox_check(void)
 
 /* security guards to force Vim's sandbox restrictions on MzScheme level */
     static Scheme_Object *
-sandbox_file_guard(int argc, Scheme_Object **argv)
+sandbox_file_guard(int argc UNUSED, Scheme_Object **argv)
 {
     if (sandbox)
     {
@@ -3140,7 +3368,7 @@ sandbox_file_guard(int argc, Scheme_Object **argv)
 }
 
     static Scheme_Object *
-sandbox_network_guard(int argc, Scheme_Object **argv)
+sandbox_network_guard(int argc UNUSED, Scheme_Object **argv UNUSED)
 {
     return scheme_void;
 }
