@@ -443,7 +443,7 @@ getConnInfo(char *file, char **host, char **port, char **auth)
     FILE *fp;
     char_u buf[BUFSIZ];
     char_u *lp;
-    char_u *nl;
+    char_u *nlp;
 #ifdef UNIX
     struct stat	st;
 
@@ -472,8 +472,8 @@ getConnInfo(char *file, char **host, char **port, char **auth)
     /* Read the file. There should be one of each parameter */
     while ((lp = (char_u *)fgets((char *)buf, BUFSIZ, fp)) != NULL)
     {
-	if ((nl = vim_strchr(lp, '\n')) != NULL)
-	    *nl = 0;	    /* strip off the trailing newline */
+	if ((nlp = vim_strchr(lp, '\n')) != NULL)
+	    *nlp = 0;	    /* strip off the trailing newline */
 
 	if (STRNCMP(lp, "host=", 5) == 0)
 	{
@@ -1395,7 +1395,7 @@ nb_do_cmd(
     int		cmdno,
     char_u	*args)	    /* points to space before arguments or NUL */
 {
-    int		doupdate = 0;
+    int		do_update = 0;
     long	off = 0;
     nbbuf_T	*buf = nb_get_buf(bufno);
     static int	skip = 0;
@@ -1600,7 +1600,7 @@ nb_do_cmd(
 							last.lnum, last.col));
 		del_from_lnum = first.lnum;
 		del_to_lnum = last.lnum;
-		doupdate = 1;
+		do_update = 1;
 
 		/* Get the position of the first byte after the deleted
 		 * section.  "next" is NULL when deleting to the end of the
@@ -1740,7 +1740,7 @@ nb_do_cmd(
 		int	added = 0;
 		int	oldFire = netbeansFireChanges;
 		int	old_b_changed;
-		char_u	*nl;
+		char_u	*nlp;
 		linenr_T lnum;
 		linenr_T lnum_start;
 		pos_T	*pos;
@@ -1777,11 +1777,11 @@ nb_do_cmd(
 		lnum = lnum_start;
 
 		/* Loop over the "\n" separated lines of the argument. */
-		doupdate = 1;
+		do_update = 1;
 		while (*args != NUL)
 		{
-		    nl = vim_strchr(args, '\n');
-		    if (nl == NULL)
+		    nlp = vim_strchr(args, '\n');
+		    if (nlp == NULL)
 		    {
 			/* Incomplete line, probably truncated.  Next "insert"
 			 * command should append to this one. */
@@ -1789,13 +1789,13 @@ nb_do_cmd(
 		    }
 		    else
 		    {
-			len = nl - args;
+			len = nlp - args;
 
 			/*
 			 * We need to detect EOL style, because the commands
 			 * use a character offset.
 			 */
-			if (nl > args && nl[-1] == '\r')
+			if (nlp > args && nlp[-1] == '\r')
 			{
 			    ff_detected = EOL_DOS;
 			    --len;
@@ -1814,7 +1814,8 @@ nb_do_cmd(
 
 			/* Insert halfway a line.  For simplicity we assume we
 			 * need to append to the line. */
-			newline = alloc_check((unsigned)(STRLEN(oldline) + len + 1));
+			newline = alloc_check(
+				       (unsigned)(STRLEN(oldline) + len + 1));
 			if (newline != NULL)
 			{
 			    STRCPY(newline, oldline);
@@ -1826,14 +1827,15 @@ nb_do_cmd(
 		    {
 			/* Append a new line.  Not that we always do this,
 			 * also when the text doesn't end in a "\n". */
-			ml_append((linenr_T)(lnum - 1), args, (colnr_T)(len + 1), FALSE);
+			ml_append((linenr_T)(lnum - 1), args,
+						   (colnr_T)(len + 1), FALSE);
 			++added;
 		    }
 
-		    if (nl == NULL)
+		    if (nlp == NULL)
 			break;
 		    ++lnum;
-		    args = nl + 1;
+		    args = nlp + 1;
 		}
 
 		/* Adjust the marks below the inserted lines. */
@@ -1992,7 +1994,7 @@ nb_do_cmd(
 		EMSG("E640: invalid buffer identifier in initDone");
 		return FAIL;
 	    }
-	    doupdate = 1;
+	    do_update = 1;
 	    buf->initDone = TRUE;
 	    nb_set_curbuf(buf->bufp);
 #if defined(FEAT_AUTOCMD)
@@ -2081,7 +2083,7 @@ nb_do_cmd(
 					     ECMD_HIDE + ECMD_OLDBUF, curwin);
 	    buf->bufp = curbuf;
 	    buf->initDone = TRUE;
-	    doupdate = 1;
+	    do_update = 1;
 #if defined(FEAT_TITLE)
 	    maketitle();
 #endif
@@ -2109,7 +2111,7 @@ nb_do_cmd(
 		exarg.forceit = FALSE;
 		dosetvisible = TRUE;
 		goto_buffer(&exarg, DOBUF_FIRST, FORWARD, buf->bufp->b_fnum);
-		doupdate = 1;
+		do_update = 1;
 		dosetvisible = FALSE;
 
 #ifdef FEAT_GUI
@@ -2309,7 +2311,7 @@ nb_do_cmd(
 						     buf->bufp->b_fnum, TRUE);
 	    buf->bufp = NULL;
 	    buf->initDone = FALSE;
-	    doupdate = 1;
+	    do_update = 1;
 /* =====================================================================*/
 	}
 	else if (streq((char *)cmd, "setStyle")) /* obsolete... */
@@ -2400,7 +2402,7 @@ nb_do_cmd(
 		return FAIL;
 	    }
 
-	    doupdate = 1;
+	    do_update = 1;
 
 	    cp = (char *)args;
 	    serNum = strtol(cp, &cp, 10);
@@ -2448,7 +2450,7 @@ nb_do_cmd(
 		nbdebug(("    invalid buffer identifier in removeAnno\n"));
 		return FAIL;
 	    }
-	    doupdate = 1;
+	    do_update = 1;
 	    cp = (char *)args;
 	    serNum = strtol(cp, &cp, 10);
 	    args = (char_u *)cp;
@@ -2493,7 +2495,7 @@ nb_do_cmd(
 	    len = strtol(cp, NULL, 10);
 	    args = (char_u *)cp;
 	    pos = off2pos(buf->bufp, off);
-	    doupdate = 1;
+	    do_update = 1;
 	    if (!pos)
 		nbdebug(("    no such start pos in %s, %ld\n", cmd, off));
 	    else
@@ -2555,7 +2557,7 @@ nb_do_cmd(
 	    inAtomic = 0;
 	    if (needupdate)
 	    {
-		doupdate = 1;
+		do_update = 1;
 		needupdate = 0;
 	    }
 /* =====================================================================*/
@@ -2636,18 +2638,18 @@ nb_do_cmd(
 	 * Unrecognized command is ignored.
 	 */
     }
-    if (inAtomic && doupdate)
+    if (inAtomic && do_update)
     {
 	needupdate = 1;
-	doupdate = 0;
+	do_update = 0;
     }
 
     /*
      * Is this needed? I moved the netbeans_Xt_connect() later during startup
      * and it may no longer be necessary. If its not needed then needupdate
-     * and doupdate can also be removed.
+     * and do_update can also be removed.
      */
-    if (buf != NULL && buf->initDone && doupdate)
+    if (buf != NULL && buf->initDone && do_update)
     {
 	update_screen(NOT_VALID);
 	setcursor();
