@@ -19,10 +19,6 @@
 # undef Window	/* Amiga has its own Window definition */
 #endif
 
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>
-#endif
-
 #undef TRUE		/* will be redefined by exec/types.h */
 #undef FALSE
 
@@ -1026,7 +1022,7 @@ mch_get_shellsize()
 
     /* insure longword alignment */
 #ifdef __amigaos4__
-    if(!(id = AllocDosObject(DOS_INFODATA, 0)))
+    if (!(id = AllocDosObject(DOS_INFODATA, 0)))
 	goto out;
 #else
     id = (struct InfoData *)(((long)id_a + 3L) & ~3L);
